import datetime, traceback, copy
import adagenes.tools
import adagenes.tools.module_requests as req
from onkopus.conf import read_config as config
from adagenes.tools import generate_variant_dictionary
import onkopus as op


class DGIdbClient:

    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.url_pattern = config.dgidb_src
        self.srv_prefix = config.dgidb_srv_prefix
        self.extract_keys = config.dgidb_keys
        self.info_lines = config.dgidb_info_lines
        self.error_logfile = error_logfile

        self.qid_key = "q_id"
        if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
            self.qid_key = "q_id_hg19"

    def process_data(self, vcf_lines, gene_request=False):

        qid_gene_name_dc = {}
        qid_list = []
        for qid in vcf_lines.keys():
            variant_data = vcf_lines[qid]
            if ("UTA_Adapter" in variant_data) and (gene_request is False):
                if ("gene_name" in variant_data["UTA_Adapter"]) and (
                        "variant_exchange" in variant_data["UTA_Adapter"]):
                    qid_gene_name_dc[variant_data["UTA_Adapter"]["gene_name"]] = qid
                    qid_list.append(variant_data["UTA_Adapter"]["gene_name"])
            else:
                qid_list.append(qid)
        q_lists = list(op.tools.divide_list(copy.deepcopy(qid_list), chunk_size=100))

        for q_list in q_lists:
            q = ",".join(q_list)

            #if input_format == 'vcf':
            #    keys = [config.uta_adapter_srv_prefix + config.concat_char + config.uta_genomic_keys[0]]
            #    annotations = adagenes.tools.parse_vcf.extract_annotations_vcf(vcf_lines, keys)
            #else:
            #    keys = [config.uta_genomic_keys[0]]
            #    annotations = adagenes.tools.parse_vcf.extract_annotations_json(vcf_lines, config.uta_adapter_srv_prefix, keys)
            #qid_list = list(vcf_lines.keys())

            #while True:
            #max_length = int(config.config["DEFAULT"]["MODULE_BATCH_SIZE"])
            #if max_length > len(qid_list):
            #    max_length = len(qid_list)
            #qids_partial = qid_list[0:max_length]
            #qids_partial = adagenes.tools.filter_alternate_alleles(qids_partial)
            #genompos_str = ','.join(qids_partial)
            #gene_names_partial = \
            #    adagenes.tools.parse_vcf.extract_annotations_json_part(vcf_lines, config.uta_adapter_srv_prefix,[config.uta_genomic_keys[0]],
            ##                                                         qids_partial)[config.uta_genomic_keys[0]]
            #gene_names_str = ",".join(gene_names_partial)
            query = 'genes=' + q#gene_names_str
            query = '?' + query
            #dc = adagenes.tools.parse_genomic_data.generate_dictionary(gene_names_partial,qids_partial)

            try:
                json_body = req.get_connection(query, self.url_pattern, "hg38")

                for gene_name in json_body.keys():

                        if gene_request is False:
                            qid = qid_gene_name_dc[gene_name]
                        else:
                            qid = gene_name

                        try:
                            vcf_lines[qid][self.srv_prefix] = json_body[gene_name]
                        except:
                            if self.error_logfile is not None:
                                cur_dt = datetime.datetime.now()
                                date_time = cur_dt.strftime("%m/%d/%Y, %H:%M:%S")
                                print(cur_dt, ": error processing variant response: ", qid, ';', traceback.format_exc(), file=self.error_logfile+str(date_time)+'.log')
                            else:
                                print(cur_dt, ": error processing variant response: ", qid, ';', traceback.format_exc())
            except:
                print(": error processing variant response: ;", traceback.format_exc())
                #if self.error_logfile is not None:
                #    print("error processing request: ", annotations, file=self.error_logfile+str(date_time)+'.log')

            #for i in range(0,max_length):
            #    #del gene_names[0] #gene_names.remove(qid)
            #    #del variant_exchange[0]  #variant_exchange.remove(qid)
            #    del qid_list[0] # qid_list.remove(qid)
            #if len(qid_list) == 0:
            #    break

        return vcf_lines



import traceback, datetime, json
import adagenes.tools.module_requests as req
from onkopus.conf import read_config as config
import adagenes as ag


class MolecularFeaturesClient:

    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.info_lines= config.molecular_features_info_lines
        self.url_pattern = config.molecular_features_src
        self.srv_prefix = config.molecular_features_srv_prefix
        self.response_keys = config.molecular_features_response_keys
        self.extract_keys = config.molecular_features_keys

        self.qid_key = "q_id"
        if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
            self.qid_key = "q_id_hg19"
        self.error_logfile = error_logfile

    def process_data(self, biomarker_data, tumor_type=None):
        """

        :param biomarker_data:
        :param tumor_type:
        :return:
        """
        try:
            qid_list = []
            genes = []
            variants = []
            for var in biomarker_data.keys():
                if "UTA_Adapter" in biomarker_data[var]:
                    genes.append(biomarker_data[var]["UTA_Adapter"]["gene_name"])
                    variants.append(biomarker_data[var]["UTA_Adapter"]["variant_exchange"])
                    qid_list.append(var)

            qid_lists_query = ag.tools.split_list(qid_list)
            genes_lists_query = ag.tools.split_list(genes)
            variants_lists_query = ag.tools.split_list(variants)

            for qlist, glist, vlist in zip(qid_lists_query, genes_lists_query, variants_lists_query):
                q_genes = ",".join(glist)
                q_variants = ",".join(vlist)
                q_genompos = ",".join(qlist)
                q = "?genompos=" + q_genompos + "&gene=" + q_genes + "&variant=" + q_variants

                res = req.get_connection(q, self.url_pattern, self.genome_version)

                for var in res.keys():
                    biomarker_data[var]["molecular_features"] = res[var]["molecular_features"]

        except:
            if self.error_logfile is not None:
                cur_dt = datetime.datetime.now()
                date_time = cur_dt.strftime("%m/%d/%Y, %H:%M:%S")
                print("error processing request: ", biomarker_data, file=self.error_logfile+str(date_time)+'.log')
            else:
                print(": error processing variant response: ;", traceback.format_exc())

        return biomarker_data



import unittest, copy
import onkopus as op

class MolFeatAnnotationTestCase(unittest.TestCase):

    def test_molfeat_client(self):
        #genome_version = 'hg19'
        genome_version = 'hg38'

        #data = {"chr17:7681744T>C": {}, "chr10:8115913C>T": {}}
        data = {"chr7:140753336A>T":{}}

        variant_data = op.UTAAdapterClient(genome_version=genome_version).process_data(data)
        variant_data = op.MolecularFeaturesClient(
            genome_version=genome_version).process_data(variant_data)

        #print("Response ",variant_data)
        self.assertEqual(variant_data["chr7:140753336A>T"]["molecular_features"]["aromaticity_alt"],0,"")




import unittest
import onkopus as op


class TestGeneFusionClient(unittest.TestCase):

    def test_gene_fusion_client(self):
        genome_version = 'hg38'
        data = {"chr7:47344754-chr7:55157662": {}}
        variant_data = op.CCSGeneFusionClient(
            genome_version=genome_version).process_data(data)
        print("Response ", variant_data)





tag = "develop"

available_modules = ["dbsnp","clinvar","ccs","ccs_gene","alphamissense","revel","mvp","primateai","gencode","dbnsfp","civic","oncokb",
                     "metakb","loftool","vuspredict","ccs_protein","drug_classification","treatments",""]

module_ids = {
    "dbsnp": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/snpservice",
    "clinvar": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/clinvar-service",
    "uta": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/utaadapter",
    "fathmm": "docker.gitlab.gwdg.de/ukeb/mtb/vus-predict/fathmmadapter",
    "m3d": "docker.gitlab.gwdg.de/ukeb/mtb/vus-predict/m3dadapter",
    "uniprot": "docker.gitlab.gwdg.de/ukeb/mtb/vus-predict/uniprotfetcher",
    "vep": "docker.gitlab.gwdg.de/ukeb/mtb/vus-predict/vepadapter",
    "vus-predict": "docker.gitlab.gwdg.de/ukeb/mtb/vus-predict/vuspredict",
    "revel": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/variant-scores/revel-adapter",
    "loftool": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/variant-scores/loftool-service",
    "onkopus-server": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus/onkopus-server",
    "onkopus-web": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus/onkopus-web-frontend",
    "metakb": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/treatment-databases/metakb-adapter",
    "oncokb": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/treatment-databases/oncokb-adapter",
    "civic": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/treatment-databases/civic-adapter",
    "dbnsfp": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/dbnsfp-adapter",
    "primateai": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/variant-scores/primateai-adapter",
    "onkopus-database": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus/onkopus-database",
    "onkopus-websocket-server": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus/onkopus-websocket-server",
}


containers = {
    "snpservice": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/snpservice",
    "clinvar-service": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/clinvar-service",
    "uta-database": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/uta-database",
    "uta-adapter": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/utaadapter",
    "fathmm-adapter": "docker.gitlab.gwdg.de/ukeb/mtb/vus-predict/fathmmadapter",
    "fathmm-db": "docker.gitlab.gwdg.de/ukeb/mtb/vus-predict/fathmm-db",
    "m3d-adapter": "docker.gitlab.gwdg.de/ukeb/mtb/vus-predict/m3dadapter",
    "uniprot-fetcher": "docker.gitlab.gwdg.de/ukeb/mtb/vus-predict/uniprotfetcher",
    "vep-adapter": "docker.gitlab.gwdg.de/ukeb/mtb/vus-predict/vepadapter",
    "vus-predict": "docker.gitlab.gwdg.de/ukeb/mtb/vus-predict/vuspredict",
    "revel-adapter": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/variant-scores/revel-adapter",
    "loftool-service": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/variant-scores/loftool-service",
    "onkopus-server": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus/onkopus-server",
    "onkopus-web": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus/onkopus-web-frontend",
    "metakb-adapter": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/treatment-databases/metakb-adapter",
    "oncokb-adapter": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/treatment-databases/oncokb-adapter",
    "civic-adapter": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/treatment-databases/civic-adapter",
    "civic-database": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/treatment-databases/civic-db",
    "dbnsfp-adapter": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/dbnsfp-adapter",
    "primateai-adapter": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus-modules/variant-scores/primateai-adapter",
    "onkopus-database": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus/onkopus-database",
    "onkopus-websocket-server": "docker.gitlab.gwdg.de/ukeb/mtb/onkopus/onkopus-websocket-server"
}




import unittest
import onkopus


class InterpreterTestCase(unittest.TestCase):

    def test_interpreter_client(self):
        genome_version = 'hg38'

        data = {"chr7:140753336A>T": {}, "chr12:25245350C>T": {}}

        variant_data = onkopus.onkopus_clients.uta_adapter_client.UTAAdapterClient(
            genome_version=genome_version).process_data(data)
        print("uta response", variant_data)

        client = onkopus.onkopus_clients.REVELClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)
        client = onkopus.onkopus_clients.MVPClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)
        client = onkopus.onkopus_clients.DBNSFPClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)

        variant_data = onkopus.onkopus_clients.InterpreterClient(
            genome_version=genome_version).process_data(variant_data, tumor_type="Melanoma")

        print("Interpreter response ",variant_data)



import requests
import adagenes.conf.read_config as conf_reader


files = [
]

import os

for file in os.listdir(os.getenv("source_data_dir")):
    if os.path.isdir(os.getenv("source_data_dir") + "/" + file):
        for infile in os.listdir(os.getenv("source_data_dir") + "/" + file):
            if infile.endswith(".maf"):
                files.append(os.getenv("source_data_dir") + "/" + file + "/" + infile)
# print("files: ",files)

base_url = conf_reader.__MODULE_PROTOCOL__ + "://" + conf_reader.__MODULE_SERVER__ + ':10100'

for file in files:
    data = {}
    data['data_dir'] = 'loc'
    data['file_src'] = file

    headers= { 'Content-Type':'multipart/form-data' }
    url = base_url + '/onkopus-server/v1/upload_variant_data'
    print(url)
    response = requests.post(url, files={}, data=data)
    print(response.text)

    pid = response.text

    # add patient to MTB
    mtb_id = ""
    url = base_url + '/onkopus-server/v1/updateMTB?action=a&mtb_id=' + mtb_id + '&pid=' + str(pid)
    print(url)
    response = requests.get(url)
    print(response.json())

    # annotate files
    url = base_url + '/onkopus-server/v1/analyze_variants?data_dir=loc&id=' + str(pid)
    print(url)
    response = requests.get(url)
    print(response.json())




import unittest
import adagenes as ag
import onkopus.onkopus_clients


class UTAAdapterGeneAnnotationTestCase(unittest.TestCase):

    def test_uta_adapter_genetogenomic_client(self):
        genome_version = 'hg38'
        data = {"NRAS:Q61L": {}, "TP53:R282W": {}}
        bframe = ag.BiomarkerFrame(data)
        print(bframe.data)
        variant_data = onkopus.onkopus_clients.CCSGeneToGenomicClient(
            genome_version=genome_version).process_data(bframe.data, data_type="p")
        print("Response ",variant_data)
        qids = ["chr1:114713908T>A", "chr17:7673776G>A"]
        self.assertListEqual(list(variant_data.keys()), qids, "Error UTA adapter GeneToGenomic")
        self.assertEqual(variant_data["chr1:114713908T>A"]["mutation_type"],"snv","")

    def test_uta_adapter_client_batch(self):
        genome_version = 'hg38'

        #infile='../../test_files/somaticMutations.l100.vcf'
        infile = '../../test_files/somaticMutations.vcf'
        #onkopus.annotate_file(file, file+'.clinvar', 'clinvar', genome_version=genome_version)

        #data = onkopus.read_file(file, genome_version=genome_version)

        #infile="../../test_files/somaticMutations.ln_12.vcf"
        #infile="../../test_files/medium_input.vcf"
        #infile="../../test_files/99_input.vcf"
        #data = onkopus.VCFReader(genome_version).read_file(infile=infile)
        #data = onkopus.read_file(infile)
        #data.data = onkopus.onkopus_clients.UTAAdapterClient(genome_version=genome_version).process_data(data.data)
        #variant_data = onkopus.onkopus_clients.CCSGeneToGenomicClient(
        #    genome_version=genome_version).process_data(data.data)
        #print("Response ",variant_data)



import adagenes
import onkopus as op


def annotate(bframe:adagenes.BiomarkerFrame,
             genome_version=None,
             oncokb_key=None,
             lo_hg19=None,
             lo_hg38=None,
             lo_t2t=None,
             tumor_type=None) -> adagenes.BiomarkerFrame:
    """
    Runs the full Onkopus annotation pipeline to annotate biomarkers

    :param tumor_type:
    :param lo_t2t:
    :param lo_hg38:
    :param lo_hg19:
    :param oncokb_key:
    :param genome_version:
    :param bframe:
    :return:
    """
    if genome_version is not None:
        bframe.genome_version = genome_version

    # Recognize biomarkers
    bframe = adagenes.recognize_biomarker_types(bframe)

    # Map biomarkers on protein and transcript level to genomic level (MANE Select)
    bframe = op.ProteinToGenomic().process_data(bframe)

    # Liftover
    target_genome = None
    if bframe.genome_version != "hg38":
        target_genome = "hg38"
    bframe = adagenes.liftover(bframe, target_genome=target_genome)

    # Annotate biomarkers
    bframe.data = op.annotate_snvs(bframe.data,genome_version=bframe.genome_version,oncokb_key=oncokb_key,
                                   lo_hg19=lo_hg19,lo_hg38=lo_hg38,lo_t2t=lo_t2t, tumor_type=tumor_type)
    bframe.data = op.annotate_indels(bframe.data,genome_version=bframe.genome_version,oncokb_key=oncokb_key,tumor_type=tumor_type)
    bframe.data = op.annotate_fusions(bframe.data,genome_version=bframe.genome_version,tumor_type=tumor_type)
    bframe.data = op.annotate_genes(bframe.data, genome_version=bframe.genome_version,tumor_type=tumor_type)

    return bframe




import copy, gzip, os, traceback, time
from multiprocessing import Process, Pool
import copy
import adagenes
import onkopus.onkopus_clients
from threading import Thread
from onkopus.processing.threads import ThreadWithReturnValue


def parallel_requests(annotated_data, genome_version="hg38",oncokb_key=None):
    """
    Annotates a biomarker frame with parallelized Onkopus requests

    :param annotated_data:
    :param genome_version:
    :param oncokb_key: Private OncoKB account access token (Required for including OncoKB results)
    :return:
    """
    start_time = time.time()

    task1 = onkopus.onkopus_clients.GENCODEGeneNameClient(genome_version=genome_version).process_data
    task2 = onkopus.onkopus_clients.CIViCGeneClient(genome_version=genome_version).process_data
    task3 = onkopus.onkopus_clients.UTAAdapterProteinSequenceClient(genome_version=genome_version).process_data
    task4 = onkopus.COSMICGeneCensusClient(genome_version=genome_version).process_data
    task5 = onkopus.DGIdbClient(genome_version=genome_version).process_data
    task6 = onkopus.GeneExpressionClient().process_data

    annotated_data_oncokb = copy.deepcopy(annotated_data)
    annotated_data_oncokb["oncokbkey"] = oncokb_key

    t1 = ThreadWithReturnValue(target=task1, args=[annotated_data])
    t2 = ThreadWithReturnValue(target=task2, args=[annotated_data])
    t3 = ThreadWithReturnValue(target=task3, args=[annotated_data])
    t4 = ThreadWithReturnValue(target=task4, args=[annotated_data,True])
    t5 = ThreadWithReturnValue(target=task5, args=[annotated_data,True])
    t6 = ThreadWithReturnValue(target=task6, args=[annotated_data, True])

    t1.start()
    t2.start()
    t3.start()
    t4.start()
    t5.start()
    t6.start()

    data1 = t1.join()
    data2 = t2.join()
    data3 = t3.join()
    data4 = t4.join()
    data5 = t5.join()
    data6 = t6.join()

    annotated_data = adagenes.merge_dictionaries(annotated_data, data1)
    annotated_data = adagenes.merge_dictionaries(annotated_data, data2)
    annotated_data = adagenes.merge_dictionaries(annotated_data, data3)
    annotated_data = adagenes.merge_dictionaries(annotated_data, data4)
    annotated_data = adagenes.merge_dictionaries(annotated_data, data5)
    annotated_data = adagenes.merge_dictionaries(annotated_data, data6)

    stop_time = time.time() - start_time
    print("Time for parallel annotation requests: ",stop_time)

    return annotated_data




import unittest
import onkopus

class TestOnkopusIOFunctions(unittest.TestCase):

    def test_get_onkopus_clients(self):
        module = "clinvar"
        genome_version = "hg19"

        client = onkopus.get_onkopus_client(module, genome_version)




import traceback,copy
import onkopus as op
import adagenes.tools.module_requests as req
import adagenes.tools.parse_genomic_data
from onkopus.conf import read_config as config
from adagenes.tools import split_gene_name
import onkopus.tools
import adagenes as ag


def filter_empty_variants(q):
    """
    Filters all variants without a gene name and protein change from the query string

    :param q:
    :return:
    """
    q_new = ""
    q_elements = q.split(",")
    for el in q_elements:
        if (el == "") or (el == ":"):
            pass
        else:
            q_new += el + ","
    q_new = q_new.rstrip(",")
    return q_new


def generate_variant_str_from_gene_names_prot_change(vcf_lines):
    """
    Generates a GeneToGenomic request to the Coordinates Converter service by extracting the protein information from the biomarker
    keys, where keys should be of the format [gene_symbol]:[aa_exchange] (e.g. BRAF:V600E)

    :param vcf_lines:
    :return:
    """
    q = ""
    for variant in vcf_lines.keys():
        resp = split_gene_name(variant)
        if resp:
            gene, variant_exchange = resp[0], resp[1]
            q += gene+":"+variant_exchange+","
    q = q.rstrip(",")
    return q


def generate_variant_str_from_data_in_json(vcf_lines,q_list):
    """
    Generates a GeneToGenomic request to the Coordinates Converter by extracting gene names and amino acid exchange from the biomarker data.
    Usable if a biomarker frame has already been annotated with the CCS GenomicToGene service and should be enriched with additional
    data by the GeneToGenomic service

    :param vcf_lines:
    :return:
    """
    q = ""
    for variant in q_list:
        if config.uta_adapter_srv_prefix in vcf_lines[variant]:
            if "gene_name" in vcf_lines[variant][config.uta_adapter_srv_prefix]:
                gene = vcf_lines[variant][config.uta_adapter_srv_prefix]["gene_name"]
                aa_exchange = vcf_lines[variant][config.uta_adapter_srv_prefix]["variant_exchange"]
                q += gene + ":" + aa_exchange + ","
    q = q.rstrip(",")
    #print(q)
    return q


class CCSGeneToGenomicClient:

    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.error_logfile = error_logfile
        self.srv_prefix = config.uta_adapter_genetogenomic_srv_prefix


    def generate_request_str_of_gene_names(self, vcf_lines,input_format='json'):
        """


        :param vcf_lines:
        :param input_format:
        :return:
        """

        #print("extract data: ",vcf_lines)
        variant_list=[]

        if input_format == 'vcf':
            keys = [config.uta_adapter_srv_prefix + config.concat_char + config.uta_genomic_keys[0],
                    config.uta_adapter_srv_prefix + config.concat_char + config.uta_genomic_keys[0]]
            annotations = adagenes.tools.parse_vcf.extract_annotations_vcf(vcf_lines, keys)
        elif input_format == 'tsv':
            keys = [config.uta_genomic_keys[0], config.uta_genomic_keys[1]]
            annotations = adagenes.tools.parse_vcf.extract_annotations_json(vcf_lines,
                                                                          config.uta_adapter_genetogenomic_srv_prefix, keys)
        else:
            keys = [config.uta_genomic_keys[0], config.uta_genomic_keys[1]]
            annotations = adagenes.tools.parse_vcf.extract_annotations_json(vcf_lines, config.uta_adapter_srv_prefix, keys)

        gene_names = annotations[keys[0]]
        variants = annotations[keys[1]]
        for i in range(0,len(gene_names)):
            variant_list.append(gene_names[i]+":"+variants[i])

        #print(variant_list)
        variant_str = ','.join(variant_list)
        variant_str = filter_empty_variants(variant_str)
        #print("req",variant_str)
        return variant_str, variant_list

    def generate_genome_locations_as_keys(self, gene_data):

        annotated_data = {}
        for gene_name, value in gene_data.items():

            # extract genomic locations
            if 'results_string' in value:
                results_string = value['results_string']
                chr, ref_seq, pos, ref, alt = adagenes.tools.parse_genomic_data.parse_genome_position(results_string)
                genompos = "chr" + chr + ":" + pos + ref + ">" + alt

                annotated_data[genompos] = {}
                annotated_data[genompos][config.uta_adapter_genetogenomic_srv_prefix] = value
                annotated_data[genompos]['variant_data'] = gene_data[gene_name]['variant_data']
            else:
                pass

        return annotated_data

    def process_data(self, vcf_lines,input_format='json',data_type='p'):
        """
        Extracts gene names and protein change from biomarker data and retrieves genomic data from the Coordinates Converter service

        :param gene_data:
        :param input_format:
        :return:
        """
        # generate query string
        #variant_str, variant_list = self.generate_request_str_of_gene_names(gene_data,input_format=input_format)
        qid_list = list(vcf_lines.keys())
        q_lists = list(op.tools.divide_list(copy.deepcopy(qid_list), chunk_size=100))

        if data_type == 'p':
            annotated_data = {}
        elif data_type == 'g':
            annotated_data = vcf_lines
        else:
            annotated_data = vcf_lines

        for q_list in q_lists:
            if data_type == 'p':
                #variant_str = generate_variant_str_from_gene_names_prot_change(vcf_lines)

                plist = []
                for var in q_list:

                    if "type" not in vcf_lines[var]:
                        vcf_lines[var] = ag.TypeRecognitionClient().process_data( { var: vcf_lines[var]} )[var]
                    var_type = vcf_lines[var]["type"]
                    if var_type == "p":
                        plist.append(var)
                    else:
                        annotated_data[var] = vcf_lines[var]

                variant_str = ",".join(plist)
            elif data_type == 'g':
                variant_str = generate_variant_str_from_data_in_json(vcf_lines,q_list)
            else:
                variant_str = generate_variant_str_from_data_in_json(vcf_lines, q_list)

            try:
                if variant_str != '':
                    json_body = req.get_connection(variant_str,config.uta_adapter_genetogenomic_src,self.genome_version)
                    for item in json_body:

                        if (item["data"] is not None) and not (isinstance(item["data"],str)):
                            for res in item["data"]:
                                    if res != "Error":
                                        try:
                                            qid = item["header"]["qid"]
                                            results_string = res['results_string']

                                            if qid not in vcf_lines.keys():
                                                preresult = {}
                                            else:
                                                preresult = vcf_lines[qid]

                                            annotated_data = onkopus.tools.protein_to_genomic(annotated_data,results_string,res,preresult,
                                                                                     self.srv_prefix, genome_version=self.genome_version)

                                        except:
                                            print("Error retrieving genomic UTA response ",res)
                                            print(traceback.format_exc())
                        else:
                            qid = item["header"]["qid"]
                            gene,protein=qid.split(":")

                            if qid not in annotated_data:
                                annotated_data[qid] = {}

                            if config.variant_data_key not in annotated_data[qid]:
                                annotated_data[qid][config.variant_data_key] = {}
                            annotated_data[qid][config.variant_data_key]["gene"] = gene
                            annotated_data[qid][config.variant_data_key]["variant_exchange"] = protein
                            annotated_data[qid][config.variant_data_key]["type"] = "unidentified"
                            annotated_data[qid][config.variant_data_key]["status"] = "error"
                            annotated_data[qid][config.variant_data_key]["status_msg"] = item["data"]
            except:
                print("error: genomic to gene")
                print(traceback.format_exc())

        return annotated_data



import copy
import traceback, datetime, json
import adagenes.tools.module_requests as req
from onkopus.conf import read_config as conf_reader
import requests
from json import JSONDecodeError
import onkopus as op


class ProteinFeatureClient:

    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.info_lines= conf_reader.onkopus_plots_info_lines
        self.url_pattern = conf_reader.onkopus_plots_src
        self.srv_prefix = conf_reader.onkopus_plots_srv_prefix
        self.response_keys = conf_reader.onkopus_plots_response_keys
        self.extract_keys = conf_reader.onkopus_plots_keys

        self.qid_key = "q_id"
        if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
            self.qid_key = "q_id_hg19"
        self.error_logfile = error_logfile

    def process_data(self, data, plot=None):
            """
            Generates JSON data for a Plotly variant needleplot with added variant annotations

            :param variant_data:
            :return:
            """
            qid_gene_name_dc = {}
            qid_list=[]
            for qid in data.keys():
                variant_data = data[qid]
                if "UTA_Adapter" in variant_data:
                    if ("gene_name" in variant_data["UTA_Adapter"]) and ("variant_exchange" in variant_data["UTA_Adapter"]):
                        qid_gene_name_dc[variant_data["UTA_Adapter"]["gene_name"] + ':' + variant_data["UTA_Adapter"]["variant_exchange"]] = qid
                        qid_list.append(variant_data["UTA_Adapter"]["gene_name"] + ':' + variant_data["UTA_Adapter"]["variant_exchange"])
                else:
                    #qid_list.append(qid)
                    #print("ProteinFeatures: Could not find UTA adapter section: ", variant_data)
                    pass
            q_lists = list(op.tools.divide_list(copy.deepcopy(qid_list), chunk_size=100))

            for q_list in q_lists:
                q = ",".join(q_list)
                url = conf_reader.protein_module_features_src + "/" + q
                print(url)
                try:
                    json_obj = requests.get(url, timeout=60).json()
                    for result in json_obj:
                        if result["data"] is not None:
                            qid = qid_gene_name_dc[result["header"]["qid"]]
                            data[qid]["protein_features"] = result["data"]
                except JSONDecodeError:
                    print("Could not decode JSON: ",traceback.format_exc())
                except KeyError:
                    print("Could not parse variant: ",traceback.format_exc())

            return data



import unittest, os
import onkopus.onkopus_clients


class OncoKBAnnotationTestCase(unittest.TestCase):

    def test_oncokb_client(self):
        genome_version = 'hg38'

        data = {"chr7:140753336A>T": {}}
        key = os.getenv("ONCOKB_KEY")

        variant_data = onkopus.UTAAdapterClient(genome_version=genome_version).process_data(data)
        variant_data = onkopus.onkopus_clients.OncoKBClient(
            genome_version=genome_version).process_data(variant_data,key=key)

        if "oncokb" in variant_data["chr7:140753336A>T"]:
            print("Response ",variant_data["chr7:140753336A>T"]["oncokb"])

    #def test_oncokb_client_batch(self):
    #    __location__ = os.path.realpath(
    #        os.path.join(os.getcwd(), os.path.dirname(__file__)))
    #    genome_version = 'hg19'
    #    file= __location__ + '/../../test_files/somaticMutations.vcf'
    #    data = onkopus.read_file(file)
    #    data.data = onkopus.onkopus_clients.UTAAdapterClient(genome_version=genome_version).process_data(data.data)
    #    data.data = onkopus.onkopus_clients.OncoKBClient(
    #        genome_version=genome_version).process_data(data.data)
    #    print(data.data)



import unittest
import onkopus
"""

class TestCLIAnnotation(unittest.TestCase):


    def test_cli_annotation(self):
        infile = "./test_files/input.csv"
        outfile = infile + ".anno.csv"
        bframe = onkopus.read_file(infile)
        bframe.data = onkopus.annotate_variant_data(bframe.data)
        onkopus.write_file(outfile,bframe)

"""


import unittest
import onkopus as op


class ProteinFeaturesTestCase(unittest.TestCase):

    def test_pdb_client(self):
        genome_version = 'hg38'

        data = {"chr7:140753336A>T": {}, "chr12:25245350C>T": {}}

        variant_data = op.uta_adapter_client.UTAAdapterClient(
            genome_version=genome_version).process_data(data)
        print("uta response", variant_data)

        client = op.REVELClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)
        client = op.MVPClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)
        client = op.DBNSFPClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)
        #variant_data = onkopus.InterpreterClient(genome_version="hg38").process_data(variant_data)

        variant_data = op.PDBClient(
            genome_version=genome_version).process_data(variant_data["chr7:140753336A>T"])

    def test_pdb_client_protein(self):
        genome_version = 'hg38'

        data = {"ANKRD11:P2117L": {}}

        variant_data = op.CCSGeneToGenomicClient(genome_version=genome_version).process_data(data, data_type="p")
        variant_data = op.UTAAdapterClient(
                genome_version=genome_version).process_data(variant_data)
        print("uta response", variant_data)

        client = op.REVELClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)
        client = op.MVPClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)
        client = op.DBNSFPClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)
        # variant_data = onkopus.InterpreterClient(genome_version="hg38").process_data(variant_data)

        variant_data = op.PDBClient(
                genome_version=genome_version).process_data(variant_data[list(variant_data.keys())[0]])
        #print(variant_data)

        #print("Plot response ",variant_data[0]["data"]["PDB_file_string"])
        print(len(variant_data[0]["data"]["PDB_file_string"]))
        # print(variant_data[0]["data"]["PDB_file_string"],file=outfile)
        # outfile.close()
        self.assertEqual(len(variant_data[0]["data"]["PDB_file_string"]),1716146,"Error PDB retrieval")



import unittest
import onkopus as op


class TestGeneAnnotation(unittest.TestCase):

    def test_gene_annotation(self):
        data = {"TP53":{ "mutation_type": "gene" }}
        data = op.annotate_genes(data)
        print(data)




import adagenes
import onkopus.conf.read_config as conf_reader

def avf_to_csv(infile,outfile):
    adagenes.avf_to_csv(infile,outfile,mapping=conf_reader.tsv_mappings,labels=conf_reader.tsv_labels,ranked_labels=conf_reader.tsv_feature_ranking)




import unittest, os
import onkopus as op
import adagenes as ag


class SpecificFeaturesExportTestCase(unittest.TestCase):

    def test_specified_features_export(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + '/../test_files/somaticMutations.ln50.avf'
        outfile = open(__location__ + "/../test_files/somaticMutations.ln50.revel.csv", 'w')

        bframe = op.read_file(infile, genome_version="hg19")
        print("pre freatures ",bframe.preexisting_features)
        bframe = ag.LiftoverClient().process_data(bframe, target_genome="hg38")

        bframe.data = op.DBNSFPClient(genome_version="hg38").process_data(bframe.data)

        op.write_file(outfile, bframe, file_type="csv", export_features=["variant_data>CHROM","variant_data>POS", "variant_data>REF","variant_data>ALT", "dbnsfp>REVEL_score_aggregated_value"])



import unittest,os
import adagenes
import onkopus as op


class VariantDataExportTestCase(unittest.TestCase):

    def test_export_variant_data(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))

        data = {"chr7:140753336A>T":{}}
        genome_version = "hg38"
        bf = adagenes.BiomarkerFrame()

        bf.data = op.annotate_variant_data(data, genome_version="hg38")
        #print(data)

        outfile=__location__ + "/../test_files/test.out.csv"
        op.write_file(outfile, bf, file_type="csv")




import unittest
import onkopus


class AggregationTestCase(unittest.TestCase):

    def test_aggregator_client(self):
        genome_version = 'hg38'

        data = {"chr7:140753336A>T": {}, "chr12:25245350C>T": {}}
        variant_dc = {"0": "chr7:140753336A>T", "1": "chr12:25245350C>T"}

        variant_data = onkopus.onkopus_clients.uta_adapter_client.UTAAdapterClient(
            genome_version=genome_version).process_data(data)
        print("uta response", variant_data)

        client = onkopus.onkopus_clients.MetaKBClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)
        client = onkopus.onkopus_clients.CIViCClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)
        client = onkopus.onkopus_clients.OncoKBClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)

        # Aggregator
        variant_dc = onkopus.generate_variant_dictionary(variant_data)
        variant_data = onkopus.onkopus_clients.aggregator_client.AggregatorClient(
            genome_version=genome_version).process_data(variant_data)

        print("Aggregator response ",variant_data)



import unittest
import onkopus as op


class GEPlotTestCase(unittest.TestCase):

    def test_ge_plot_client(self):
        genome_version = 'hg38'

        data = {"chr7:140753336A>T": {}, "chr12:25245350C>T": {}}

        variant_data = op.uta_adapter_client.UTAAdapterClient(
            genome_version=genome_version).process_data(data)
        print("uta response", variant_data)

        variant_data = op.GeneExpressionPlotClient().process_data(variant_data["chr7:140753336A>T"])
        print(variant_data)
        #self.assertEqual(variant_data['BRAF']['data'][0]['marker']['color'], "lightskyblue", "")




import unittest
import onkopus as op


class ScanNetTestCase(unittest.TestCase):

    def test_bindingsite_client(self):
        genome_version = 'hg38'

        qid = "chr7:140753336A>T"
        data = {qid: {}, "chr12:25245350C>T": {}}

        variant_data = op.UTAAdapterClient(
            genome_version=genome_version).process_data(data)
        print("uta response", variant_data)

        #print(variant_data)
        #print("pdb ",pdb_data)
        gene = variant_data[qid]["UTA_Adapter"]["gene_name"]

        variant_data = op.ScanNetBindingSiteClient(
            genome_version=genome_version).process_data(gene)

        print("ScanNet response ",variant_data)


    def test_bindingsite_client_gene(self):
        genome_version = 'hg38'

        gene = "TMA7B"
        data = {gene: {"UTA_Adapter":{"gene_name": gene}}, }

        #variant_data =
        #variant_data = op.UTAAdapterClient(
        #    genome_version=genome_version).process_data(data)
        #print("uta response", variant_data)

        #pdb_data = op.PDBClient(
        #    genome_version=genome_version).process_data(variant_data[gene])

        #print(variant_data)
        #print("pdb ",pdb_data)
        #gene = variant_data[gene]["UTA_Adapter"]["gene_name"]

        variant_data = op.ScanNetBindingSiteClient(
            genome_version=genome_version).process_data(gene)

        print("ScanNet response ",variant_data)




import unittest, copy, os
import onkopus.onkopus_clients

class ClinVarAnnotationTestCase(unittest.TestCase):

    def test_clinvar_client(self):
        genome_version = 'hg19'

        data = {"chr17:7681744T>C": {}, "chr10:8115913C>T": {}, "chr10:8115914C>.": {}}

        variant_data = onkopus.onkopus_clients.ClinVarClient(
            genome_version=genome_version).process_data(data)

        print("Response ",variant_data)
        self.assertListEqual(["chr17:7681744T>C", "chr10:8115913C>T", "chr10:8115914C>."], list(variant_data.keys()), "")
        self.assertEqual('0.00001',variant_data["chr10:8115913C>T"]["clinvar"]["AF_EXAC"],"")

    def test_clinvar_client_batch(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        input_file = __location__ + "/../../test_files/somaticMutations.vcf"
        genome_version = 'hg38'
        #onkopus.annotate_file(file, file+'.clinvar', 'clinvar', genome_version=genome_version)

        data = onkopus.read_file(input_file)

        data.data = onkopus.onkopus_clients.ClinVarClient(
            genome_version=genome_version).process_data(data.data)

        print(data.data)



import unittest, os
import onkopus


class TestCLIAnnotation(unittest.TestCase):
#

    def test_cli_annotation(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + "/../test_files/somaticMutations.vcf"
        outfile = infile + ".anno.csv"
        bframe = onkopus.read_file(infile)
        bframe.data = onkopus.annotate_variant_data(bframe.data)
        onkopus.write_file(outfile,bframe)




import copy
import traceback, datetime, json
import adagenes.tools.module_requests as req
from onkopus.conf import read_config as conf_reader
import requests


def post_connection_format(biomarker_data, url, genome_version):
    """
    Requests a module over a HTTP POST request

    :param biomarker_data:
    :param url:
    :param genome_version:
    :return:
    """
    url = url.format(genome_version)
    print(url)
    r = requests.post(url, json= biomarker_data, timeout=120)
    #r = requests.post(url, data= biomarker_data, timeout=120)
    print(r.elapsed, " , ", url)
    return r.text

class ScanNetBindingSiteClient:

    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.info_lines= conf_reader.scannet_info_lines
        self.url_pattern = conf_reader.scannet_src
        self.srv_prefix = conf_reader.scannet_srv_prefix
        self.response_keys = conf_reader.scannet_response_keys
        self.extract_keys = conf_reader.scannet_keys

        self.qid_key = "q_id"
        if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
            self.qid_key = "q_id_hg19"
        self.error_logfile = error_logfile


    def process_data(self, protein):
        try:
            pdb_data = req.get_connection(protein, self.url_pattern, self.genome_version)
            #biomarker_data_json = json.loads(biomarker_data["pdb"])
            return pdb_data

        except:
            if self.error_logfile is not None:
                cur_dt = datetime.datetime.now()
                date_time = cur_dt.strftime("%m/%d/%Y, %H:%M:%S")
                print("error processing request: ", protein, file=self.error_logfile + str(date_time) + '.log')
            else:
                print(": error processing variant response: ;", traceback.format_exc())

        return pdb_data



import unittest, os
import onkopus as op
import adagenes as ag


class VariantVCFExportTestCase(unittest.TestCase):

    def test_vcf_export_patho(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + '/../test_files/somaticMutations.ln50.avf'
        outfile = __location__ + "/../test_files/somaticMutations.ln50.out.vcf"

        bframe = op.read_file(infile, genome_version="hg19")
        bframe = ag.LiftoverClient().process_data(bframe,target_genome="hg38")

        bframe.data = op.DBNSFPClient(genome_version="hg38").process_data(bframe.data)

        op.write_file(outfile, bframe)

    def test_export_file_object(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + '/../test_files/somaticMutations.ln50.avf'
        outfile = open(__location__ + "/../test_files/somaticMutations.ln50.out.fo.vcf", 'w')

        bframe = op.read_file(infile, genome_version="hg19")
        bframe = ag.LiftoverClient().process_data(bframe, target_genome="hg38")

        bframe.data = op.DBNSFPClient(genome_version="hg38").process_data(bframe.data)

        op.write_file(outfile, bframe, file_type="vcf")




import unittest, os
import onkopus
import adagenes as ag


class ProteinFeaturesTestCase(unittest.TestCase):

    def test_protein_features_client(self):
        genome_version = 'hg38'

        data = {"chr7:140753336A>T": {}, "chr12:25245350C>T": {}}

        variant_data = onkopus.onkopus_clients.uta_adapter_client.UTAAdapterClient(
            genome_version=genome_version).process_data(data)
        print("uta response", variant_data)

        client = onkopus.onkopus_clients.REVELClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)
        client = onkopus.onkopus_clients.MVPClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)
        client = onkopus.onkopus_clients.DBNSFPClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)
        #variant_data = onkopus.InterpreterClient(genome_version="hg38").process_data(variant_data)

        variant_data = onkopus.onkopus_clients.ProteinFeatureClient(
            genome_version=genome_version).process_data(variant_data)

        print("Plot response ",variant_data)
        self.assertEqual(variant_data["chr7:140753336A>T"]["protein_features"]["RSA"],0.0352112676056338,"")

    def test_protein_features_client_batch(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        genome_version = 'hg19'
        file = __location__ + '/../../test_files/somaticMutations_brca_ln250.avf'

        bframe = onkopus.read_file(file, genome_version=genome_version)
        bframe = ag.LiftoverClient(genome_version=genome_version).process_data(bframe, target_genome="hg38")
        self.assertEqual(bframe.genome_version, "hg38","")
        genome_version = bframe.genome_version

        variant_data = onkopus.onkopus_clients.uta_adapter_client.UTAAdapterClient(
            genome_version=genome_version).process_data(bframe.data)
        #print("uta response", variant_data)

        client = onkopus.onkopus_clients.REVELClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)
        client = onkopus.onkopus_clients.MVPClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)
        client = onkopus.onkopus_clients.DBNSFPClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)
        # variant_data = onkopus.InterpreterClient(genome_version="hg38").process_data(variant_data)

        variant_data = onkopus.onkopus_clients.ProteinFeatureClient(
            genome_version=genome_version).process_data(variant_data)

        #print("Plot response ", variant_data)
        qid = "chr20:32430020C>A"
        self.assertEqual(variant_data[qid]["protein_features"]["RSA"], 0.8529411764705882, "")
        self.assertEqual(variant_data[qid]["UTA_Adapter"]["gene_name"],"ASXL1", "")
        self.assertEqual(variant_data[qid]["UTA_Adapter"]["variant_exchange"], "P229T", "")



import traceback
from json import JSONDecodeError
import requests
import onkopus.conf.read_config as conf_reader


def generate_protein_plot(variant_data):
    """
    Generates JSON data for a Plotly variant needleplot

    :param variant_data:
    :return:
    """
    if "UTA_Adapter" in variant_data:
        if ("gene_name" in variant_data["UTA_Adapter"]) and ("variant_exchange" in variant_data["UTA_Adapter"]):
            q = variant_data["UTA_Adapter"]["gene_name"] + ':' + variant_data["UTA_Adapter"]["variant_exchange"]
            url = conf_reader.protein_module_src + q
            #print(url)
            try:
                graphJSON = requests.get(url, timeout=60)
                return graphJSON.json()
            except JSONDecodeError:
                print(traceback.format_exc())
                return {}
        elif "gene_name" in variant_data["UTA_Adapter"]:
            q = variant_data["UTA_Adapter"]["gene_name"]
            url = conf_reader.protein_module_src + q
            #print(url)
            try:
                graphJSON = requests.get(url, timeout=60)
                return graphJSON.json()
            except JSONDecodeError:
                print(traceback.format_exc())
                return {}
    else:
        print(variant_data)
        return {}


def generate_protein_plot_with_annotations(variant_data, module_server=conf_reader.__MODULE_SERVER__):
    """
    Generates JSON data for a Plotly variant needleplot with added variant annotations

    :param variant_data:
    :return:
    """
    if "UTA_Adapter" in variant_data:
        if ("gene_name" in variant_data["UTA_Adapter"]) and ("variant_exchange" in variant_data["UTA_Adapter"]):
            q = variant_data["UTA_Adapter"]["gene_name"] + ':' + variant_data["UTA_Adapter"]["variant_exchange"]
            #url = conf_reader.protein_module_src + q
            url = conf_reader.protein_module_annotations_src
            #print(url)
            #graphJSON = requests.get(url, timeout=60)

            data = {}
            data["web_prefix"] = conf_reader.__MODULE_PROTOCOL__ + "://" + module_server + ":" + conf_reader.__PORT_ONKOPUS_WEB__ + "/"
            data["gene_variant"] = variant_data["UTA_Adapter"]["gene_name"] + ":" + variant_data["UTA_Adapter"]["variant_exchange"]
            print(data)

            graphJSON = requests.post(url, json=data, timeout=60)
            try:
                return graphJSON.json()
            except JSONDecodeError:
                print("Could not decode JSON: ",graphJSON)
                return {}
        elif "gene_name" in variant_data["UTA_Adapter"]:
            q = variant_data["UTA_Adapter"]["gene_name"]
            url = conf_reader.protein_module_src + q
            #print(url)
            try:
                graphJSON = requests.get(url, timeout=60)
                return graphJSON.json()
            except JSONDecodeError:
                print(traceback.format_exc())
                return {}
    else:
        print(variant_data)
        return {}



import datetime, traceback, copy
import adagenes.tools.module_requests as req
import adagenes.tools.parse_vcf
from onkopus.conf import read_config as config
import adagenes.tools


class MetaKBClient:
    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.url_pattern = config.metakb_src
        self.srv_prefix = config.metakb_srv_prefix
        self.extract_keys = config.metakb_keys
        self.info_lines = config.metakb_info_lines
        self.error_logfile = error_logfile

        self.qid_key = "q_id"
        if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
            self.qid_key = "q_id_hg19"

    def process_data(self, vcf_lines, input_format='json'):
        vcf_linesf = adagenes.tools.filter_wildtype_variants(vcf_lines)

        if input_format == 'vcf':
            keys = [config.uta_adapter_srv_prefix + config.concat_char + config.uta_genomic_keys[0],
                    config.uta_adapter_srv_prefix + config.concat_char + config.uta_genomic_keys[1]]
            annotations = adagenes.tools.parse_vcf.extract_annotations_vcf(vcf_linesf, keys)
        else:
            keys = [config.uta_genomic_keys[0],
                    config.uta_genomic_keys[1]]
            annotations = adagenes.tools.parse_vcf.extract_annotations_json(vcf_linesf, config.uta_adapter_srv_prefix, keys)

        gene_names = copy.deepcopy(annotations[keys[0]])
        varexch = copy.deepcopy(annotations[keys[1]])
        qid_list = copy.deepcopy(annotations['q_id'])

        while True:
            max_length = int(config.config["DEFAULT"]["MODULE_BATCH_SIZE"])
            if max_length > len(qid_list):
                max_length = len(qid_list)
            qids_partial = qid_list[0:max_length]
            qids_partial = adagenes.tools.filter_alternate_alleles(qids_partial)

            genompos_str = ','.join(qids_partial)
            gene_names_partial = gene_names[0:max_length]
            variant_exchange_partial = varexch[0:max_length]
            variants_str=''

            for i,query_id in enumerate(qids_partial):
                variants_str += gene_names_partial[i] + ":" + variant_exchange_partial[i] + ","
            variants_str = variants_str.rstrip(",")
            query = 'variants=' + variants_str
            query += '&genompos=' + genompos_str

            try:
                    q = query
                    q += '&size=1000'
                    q += '&key=genompos'

                    json_body = req.get_connection(q, self.url_pattern, self.genome_version)

                    for genompos in json_body.keys():
                            json_obj = json_body[genompos]
                            qid = genompos

                            for k in self.extract_keys:
                                if k in json_obj:
                                    pass
                            try:
                                vcf_lines[qid][self.srv_prefix] = copy.deepcopy(json_obj)
                            except:
                                if self.error_logfile is not None:
                                    cur_dt = datetime.datetime.now()
                                    date_time = cur_dt.strftime("%m/%d/%Y, %H:%M:%S")
                                    print(cur_dt, ": error processing variant response: ", qid, ';', traceback.format_exc(), file=self.error_logfile+str(date_time)+'.log')
                                else:
                                    print(traceback.format_exc())
            except:
                    if self.error_logfile is not None:
                        print("error processing request: ", q, file=self.error_logfile+str(date_time)+'.log')
                    else:
                        print(": error processing variant response: ;", traceback.format_exc())

            for i in range(0, max_length):
                del gene_names[0]
                del varexch[0]
                del qid_list[0]
            if len(qid_list) == 0:
                break

        return vcf_lines



import os, traceback, copy, gzip
import pandas as pd
import adagenes
import adagenes.conf.read_config as conf_reader


class ClinSigWriter():
    """
    Export treatment data on the clinical significance of potential treatments
    """

    def export_feature_results_lv2(self, biomarker_data, record_path=None, meta=None) -> pd.DataFrame:
        """
        Exports biomarker data in full mode with level 2 meta data

        :param biomarker_data:
        :param outfile_src:
        :param feature:
        :param record_path:
        :param meta:
        :param sep:
        :return:
        """
        df_sum = pd.DataFrame()
        for var in biomarker_data.keys():
            if "UTA_Adapter" in biomarker_data[var].keys():
                try:
                    df = pd.json_normalize(data=biomarker_data[var], record_path=record_path, meta=meta, errors='ignore')
                    df_sum = pd.concat([df_sum, df], axis=0)
                except:
                    print(traceback.format_exc())
        return df_sum

    def fill_in_missing_keys_lv2(self,biomarker_data, struc):

        for var in biomarker_data.keys():
            for key in struc:
                if key not in biomarker_data[var]:
                    biomarker_data[var][key] = {}
                for val in struc[key]:
                    if val not in biomarker_data[var][key]:
                        biomarker_data[var][key][val] = {}

        return biomarker_data

    def write_evidence_data_to_file_all_features(self,bframe,
                                                 databases=None,
                                                 output_file=None,
                                                 format='tsv',
                                                 sep='\t'):
        """
        Writes data on clinical significance on treatments on molecular targets in an output file,
        where each row represents a potential treatment.

        :param variant_data:
        :param databases:
        :param output_file:
        :param format:
        :param sep:
        :return:
        """
        variant_data = bframe.data

        for var in variant_data.keys():
            if "onkopus_aggregator" not in variant_data[var]:
                variant_data[var]["onkopus_aggregator"] = {}
            if "merged_evidence_data" not in variant_data[var]["onkopus_aggregator"]:
                variant_data[var]["onkopus_aggregator"]["merged_evidence_data"] = {}
            if "exact_match" not in variant_data[var]["onkopus_aggregator"]["merged_evidence_data"]:
                variant_data[var]["onkopus_aggregator"]["merged_evidence_data"]["exact_match"] = []
            if "any_mutation_in_gene" not in variant_data[var]["onkopus_aggregator"]["merged_evidence_data"]:
                variant_data[var]["onkopus_aggregator"]["merged_evidence_data"]["any_mutation_in_gene"] = []

            if "UTA_Adapter" not in variant_data[var]:
                variant_data[var]["UTA_Adapter"] = {}
            if "gene_name" not in variant_data[var]["UTA_Adapter"]:
                variant_data[var]["UTA_Adapter"]["gene_name"] = ""
            if "variant_exchange" not in variant_data[var]["UTA_Adapter"]:
                variant_data[var]["UTA_Adapter"]["variant_exchange"] = ""

        #record_path = [
        #    ["onkopus_aggregator", "merged_evidence_data",
        #        "exact_match"]]
        record_path = [["onkopus_aggregator", "merged_match_types_data"]]
        meta = [["UTA_Adapter", "gene_name"], ["UTA_Adapter", "variant_exchange"]]

        # exact match
        df = self.export_feature_results_lv2(copy.copy(variant_data), record_path=record_path, meta=meta)

        # any mutation in gene
        #record_path = ["onkopus_aggregator", "merged_evidence_data",
        #        "any_mutation_in_gene"]
        #df2 = self.export_feature_results_lv2(copy.copy(variant_data), record_path=record_path, meta=meta)
        #df = pd.concat([df, df2], axis=0)

        for column in df.columns:
            df[column] = df[column].replace("\"","")
            df[column] = df[column].replace("\n", "")
            df[column] = df[column].replace("'", "")
            df[column] = df[column].replace("\t", " ")
            df[column] = df[column].replace(",", "")

        df = self.normalize_values(df)

        header = sep.join(df.columns)
        output_file.write(header + '\n')

        for _, row in df.iterrows():
            line = sep.join(map(str, row.values))  # Convert each value to string and join with commas
            output_file.write(line + '\n')  # Write data row to file


    def normalize_values(self,df):
        for i in range(0,df.shape[0]):
            drug_str = ""
            if "drugs" in df.columns:
                for drug in df.iloc[i,:].loc["drugs"]:
                    if "drug_name" in drug:
                        drug_name = str(drug["drug_name"])
                        drug_str += drug_name + ", "
                drug_str = drug_str.rstrip(", ")
            df.at[i,"drugs"] = drug_str

        return df

    def write_evidence_data_to_file(self,outfile,variant_data,sep="\t"):
        """

        :param outfile:
        :param variant_data:
        :param sep:
        :return:
        """
        fileopen = False
        if isinstance(outfile, str):
            fileopen = True
            file_name, file_extension = os.path.splitext(outfile)
            input_format_recognized = file_extension.lstrip(".")
            if input_format_recognized == "gz":
                outfile = gzip.open(outfile, 'wt')
            else:
                outfile = open(outfile, 'w')

        if isinstance(variant_data, adagenes.BiomarkerFrame):
            variant_data = variant_data.data
        bframe = adagenes.BiomarkerFrame(data=variant_data)
        self.write_evidence_data_to_file_all_features(bframe,output_file=outfile,sep=sep)

        if fileopen is True:
            outfile.close()

    def write_evidence_data_to_file2(self,variant_data, databases=None,output_file=None,format='tsv', sep='\t'):

        if isinstance(variant_data,adagenes.BiomarkerFrame):
            variant_data = variant_data.data

        if databases is None:
            databases = conf_reader.config["DEFAULT"]["ACTIVE_EVIDENCE_DATABASES"].split()

        if format == 'csv':
            sep=','

        if output_file is None:
            print("not output file given")
            return

        print(output_file)
        outfile = open(output_file, 'w')
        line = 'biomarker' + '\t' + 'disease' + '\t' + 'drugs' + '\t' + 'evidence_level' + '\t' + 'citation_id' + '\t' + 'source'
        print(line, file=outfile)

        for var in variant_data.keys():
            #print(variant_data[var].keys())

            #for db in databases:
            if 'onkopus_aggregator' in variant_data[var]:
                    if 'merged_evidence_data' in variant_data[var]['onkopus_aggregator']:
                        print(len(variant_data[var]['onkopus_aggregator']['merged_evidence_data']))
                        for match_type in variant_data[var]['onkopus_aggregator']['merged_evidence_data']:
                            for result in variant_data[var]['onkopus_aggregator']['merged_evidence_data'][match_type]:
                                #print(result)
                                drugs = result['drugs']
                                drug_name = ""
                                for drug in drugs:
                                    #print(drug)
                                    if isinstance(drug, dict):
                                        if "drug_name" in drug:
                                            drug_name += drug["drug_name"] + ","
                                drug_name = drug_name.rstrip(",")
                                line = str(result['biomarker']) + '\t' + str(result['disease']) \
                                       + '\t' + str(drug_name) + '\t' + str(result['evidence_level']) \
                                       + '\t' + str(result['citation_id']) + '\t' + str(result['source'])
                                print(line, file=outfile)
            else:
                    print("no data: ")

        outfile.close()

        def to_single_tsv_line(self, variant_data, srv_prefix, extract_keys):
            tsv_line = ''

            chr_prefix = ""
            if not variant_data[conf_reader.variant_data_key]["CHROM"].startswith("chr"):
                chr_prefix = "chr"
            tsv_line += chr_prefix + variant_data[conf_reader.variant_data_key]["CHROM"] + ':' + \
                        variant_data[conf_reader.variant_data_key]["POS"] \
                        + variant_data[conf_reader.variant_data_key]["REF"] + '>' + variant_data[conf_reader.variant_data_key][
                            "ALT"] + '\t'

            # print("write data to tsv file: ",variant_data)
            tsv_features = conf_reader.tsv_columns
            if self.features is not None:
                tsv_features = self.features

            try:
                # if srv_prefix in variant_data:

                for k in tsv_features:
                    if k in variant_data:
                        if k in conf_reader.tsv_mappings:
                            if conf_reader.tsv_mappings[k] in variant_data[k]:
                                tsv_line += str(variant_data[k][conf_reader.tsv_mappings[k]]) + '\t'
                            else:
                                tsv_line += '\t'
                        else:
                            tsv_line += str(variant_data[k]) + '\t'
                    else:
                        tsv_line += '\t'
                # else:
                #    tsv_line += '\t'
                tsv_line = tsv_line.rstrip("\t")
                # print("return ",tsv_line)
                return tsv_line.rstrip("\t")
            except:
                print(traceback.format_exc())
                return ''

    def write_to_file(self, outfile, bframe, databases=None, format="tsv", sep="\t"):
        """

        :param output_file:
        :param bframe:
        :param databases:
        :param format:
        :param sep:
        :return:
        """
        isfile=False
        if isinstance(outfile, str):
            outfile = open(outfile, 'w')
            isffle = True

        self.write_evidence_data_to_file_all_features(bframe, output_file=outfile, format=format,
                                                 sep=sep)

        if isfile is True:
            outfile.close()



import datetime, traceback, re, copy
import adagenes.tools.parse_vcf
import adagenes.tools.module_requests as req
from onkopus.conf import read_config as config

qid_key = "q_id"
error_logfile=None

class VUSPredictClient:
    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.info_lines = config.vuspredict_info_lines
        self.url_pattern = config.vuspredict_src
        self.srv_prefix = config.vuspredict_srv_prefix
        self.extract_keys = config.vuspredict_keys

        self.qid_key = "q_id"
        #if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
        #    self.qid_key = "q_id_hg19"
        self.error_logfile=error_logfile

    def process_data(self, vcf_lines, input_format='json'):
        if input_format == 'vcf':
            keys = [config.uta_adapter_srv_prefix + config.concat_char + config.uta_genomic_keys[0],
                    config.uta_adapter_srv_prefix + config.concat_char + config.uta_genomic_keys[1]]
            annotations = adagenes.tools.parse_vcf.extract_annotations_vcf(vcf_lines, keys)
        else:
            keys = [config.uta_genomic_keys[0],
                    config.uta_genomic_keys[1]]
            annotations = adagenes.tools.parse_vcf.extract_annotations_json(vcf_lines, config.uta_adapter_srv_prefix, keys)
        gene_names = copy.deepcopy(annotations[keys[0]])
        variant_exchange = copy.deepcopy(annotations[keys[1]])

        qid_list = copy.deepcopy(annotations['q_id'])
        #qid_list = copy.deepcopy(list(vcf_lines.keys()))


        while True:
            max_length = int(config.config["DEFAULT"]["MODULE_BATCH_SIZE"])
            if max_length > len(qid_list):
                max_length = len(qid_list)
            qids_partial = qid_list[0:max_length]
            qids_partial = adagenes.tools.filter_alternate_alleles(qids_partial)

            genompos_str = ','.join(qids_partial)
            gene_names_partial = \
            adagenes.tools.parse_vcf.extract_annotations_json_part(vcf_lines, config.uta_adapter_srv_prefix, [config.uta_genomic_keys[0]],
                                                                 qids_partial)[config.uta_genomic_keys[0]]
            variant_exchange_partial = adagenes.tools.parse_vcf.extract_annotations_json_part(vcf_lines,
                                                                                     config.uta_adapter_srv_prefix, [config.uta_genomic_keys[1]],
                                                                                     qids_partial)[config.uta_genomic_keys[1]]

            gene_names_str = ",".join(gene_names_partial)
            variant_exchange_str = ",".join(variant_exchange_partial)
            query = 'genesymbol=' + gene_names_str + '&variant=' + variant_exchange_str + '&genompos=' + genompos_str

            try:
                json_body = req.get_connection(query, self.url_pattern, self.genome_version)

                for json_obj in json_body:
                    if json_obj:
                        #annotations = []

                        if self.qid_key not in json_obj:
                            continue
                        qid = json_obj[self.qid_key]

                        # calculate percentage
                        if 'Score' in json_obj:
                            json_obj['score_percent'] = int( (float(json_obj['Score']) /2) * 100) + 50

                            if float(json_obj['Score']) < 0:
                                json_obj['score_percent_1of2'] = int(float(json_obj['Score']) * 100) * (-1)
                                json_obj['score_percent_2of2'] = 0
                            else:
                                json_obj['score_percent_1of2'] = 0
                                json_obj['score_percent_2of2'] = int( float(json_obj['Score']) * 100)
                            colour = "#00b0d2"
                            if float(json_obj["Score"]) < 0:
                                colour = "#ce8585"
                            json_obj['score_color'] = colour

                            json_obj["Score"] = round(float(json_obj["Score"]), 4)
                        else:
                            json_obj['score_percent_1of2'] = 0
                            json_obj['score_percent_2of2'] = 0
                            json_obj['score_color'] = "#d5d5d5"

                        for k in self.extract_keys:
                            if k in json_obj:
                                pass
                                #annotations.append('{}_{}={}'.format(self.srv_prefix, k, json_body[i][k]))

                        try:
                            json_obj.pop('q_id')
                            vcf_lines[qid][self.srv_prefix] = json_obj
                        except:
                            if self.error_logfile is not None:
                                cur_dt = datetime.datetime.now()
                                date_time = cur_dt.strftime("%m/%d/%Y, %H:%M:%S")
                                print(cur_dt, ": error processing variant response: ", qid, ';', traceback.format_exc(), file=self.error_logfile+str(date_time)+'.log')
                            else:
                                print(cur_dt, ": error processing variant response: ", qid, ';', traceback.format_exc())
            except:
                if error_logfile is not None:
                    print("error processing request: ", vcf_lines, file=error_logfile+str(date_time)+'.log')
                else:
                    traceback.print_exc()

            for i in range(0,max_length):
                del gene_names[0] #gene_names.remove(qid)
                del variant_exchange[0]  #variant_exchange.remove(qid)
                del qid_list[0] # qid_list.remove(qid)
            if len(qid_list) == 0:
                break

        return vcf_lines

    def get_genompos_with_ref_sequence(self, genompos):
        #pattern = "(chr)([0-9]+|X|Y):(g\\.|c\\.|p\\.)?([0-9]+)([A|C|T|G|>|N]+)"
        pattern = "(chr)([0-9]+|X|Y):(g\\.|c\\.|p\\.)([0-9]+)([A|C|T|G|>|N]+)"
        p = re.compile(pattern)
        m = p.match(genompos)
        if m:
            return genompos
        else:
            # add reference sequence
            pattern = "(chr)([0-9]+|X|Y):([0-9]+)([A|C|T|G|>|N]+)"
            p = re.compile(pattern)
            m = p.match(genompos)
            chr = m.group(1)
            chr_num = m.group(2)
            pos = m.group(3)
            ref = m.group(4)
            return chr + chr_num + ':' + 'g.' + pos + ref

    def get_genompos_without_ref_sequence(self, genompos):
        pattern = "(chr)([0-9]+|X|Y):([0-9]+)([A|C|T|G|>|N]+)"
        p = re.compile(pattern)
        m = p.match(genompos)
        if m:
            return genompos
        else:
            # rm reference sequence
            pattern = "(chr)([0-9]+|X|Y):(g\\.|c\\.|p\\.)([0-9]+)([A|C|T|G|>|N]+)"
            p = re.compile(pattern)
            m = p.match(genompos)
            chr = m.group(1)
            chr_num = m.group(2)
            refseq = m.group(3)
            pos = m.group(4)
            ref = m.group(5)
            return chr + chr_num + ':' + pos + ref


from .default_features_client import *
from .gencode_client import *
from .gencode_genomic_client import *
from .gencode_mane_select_transcript_client import *
from .gencode_gene_name_client import *
from .uta_adapter_protein_sequence_client import *
from .civic_gene_client import *
from .alpha_missense_client import *
from .interpreter_client import *
from .plot_client import *
from .pdb_client import *
from .protein_feature_client import *
from .cosmic_genecensus_client import *
from .dgidb_client import *
from .scannet_client import *
from .onkopus_server_client import *
from .molecular_features_client import *
from .ge_client import *
from .ge_plot_client import *



import onkopus
from onkopus.conf import read_config as config


class AllModulesClient:

    def __init__(self, genome_version):
        self.queryid = 'q_id'
        self.genome_version = genome_version
        self.srv_prefix=config.all_modules_srv_prefix
        self.extract_keys = config.all_modules_keys

    def process_data(self, biomarker_data):

        biomarker_data = onkopus.annotate_variant_data(biomarker_data)

        return biomarker_data



import datetime, requests, traceback, copy
import adagenes.tools.module_requests as req
from onkopus.conf import read_config as config
import adagenes.tools
from onkopus.onkopus_clients.processing.parse_json import parse_score_results

qid_key = "q_id"
error_logfile=None


class AlphaMissenseClient:
    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.info_lines = config.alphamissense_info_lines
        self.url_pattern = config.alphamissense_src
        self.srv_prefix = config.alphamissense_srv_prefix
        self.extract_keys = config.alphamissense_keys

        self.qid_key = "q_id"
        self.error_logfile = error_logfile

    def process_data(self, vcf_lines):
        qid_list = copy.deepcopy(list(vcf_lines.keys()))
        while True:
            max_length = int(config.config["DEFAULT"]["MODULE_BATCH_SIZE"])
            if max_length > len(qid_list):
                max_length = len(qid_list)
            qids_partial = qid_list[0:max_length]
            variants = ','.join(adagenes.tools.filter_alternate_alleles(qids_partial))

            try:
                json_body = req.get_connection(variants, self.url_pattern, self.genome_version)
                vcf_lines = parse_score_results(vcf_lines, json_body, self.srv_prefix)
            except:
                if error_logfile is not None:
                    cur_dt = datetime.datetime.now()
                    date_time = cur_dt.strftime("%m/%d/%Y, %H:%M:%S")
                    print("error processing request: ", variants, file=error_logfile+str(date_time)+'.log')

            for i in range(0, max_length):
                del qid_list[0]
            if len(qid_list) == 0:
                break

        return vcf_lines



import traceback, datetime, json
from onkopus.conf import read_config as conf_reader
import requests
from json import JSONDecodeError
import adagenes as ag


class GeneExpressionClient:

    def __init__(self, error_logfile=None):
        #self.genome_version = genome_version
        self.info_lines= conf_reader.ge_module_info_lines
        self.url_pattern = conf_reader.ge_module_src
        self.srv_prefix = conf_reader.ge_module_srv_prefix
        self.response_keys = conf_reader.ge_module_response_keys
        self.extract_keys = conf_reader.ge_module_keys

        self.qid_key = "q_id"
        self.error_logfile = error_logfile

    def process_data(self, variant_data):
            """
            Generates JSON data for a Plotly variant needleplot with added variant annotations

            :param variant_data:
            :return:
            """
            qid_gene_mapping = {}

            qid_list = []
            for var in variant_data.keys():
                if "UTA_Adapter" in variant_data[var]:
                    if ("gene_name" in variant_data[var]["UTA_Adapter"]):
                        qid_list.append(variant_data[var]["UTA_Adapter"]["gene_name"])
                        if variant_data[var]["UTA_Adapter"]["gene_name"] not in qid_gene_mapping.keys():
                            qid_gene_mapping[variant_data[var]["UTA_Adapter"]["gene_name"]] = []

                        qid_gene_mapping[variant_data[var]["UTA_Adapter"]["gene_name"]].append(var)
                elif "mutation_type" in variant_data[var]:
                    if variant_data[var]["mutation_type"] == "gene":
                        qid_list.append(var)

                        if var not in qid_gene_mapping.keys():
                            qid_gene_mapping[var] = []
                        qid_gene_mapping[var].append(var)

            qid_lists_query = ag.tools.split_list(qid_list)

            for qid_list in qid_lists_query:
                url = conf_reader.ge_module_src + "?gene=" + ",".join(qid_list)
                print(url)
                json_body = requests.get(url, timeout=60).json()
                try:
                    for gene in json_body.keys():
                        for associated_var in qid_gene_mapping[gene]:
                            variant_data[associated_var]["gtex"] = json_body[gene][0]
                except:
                    print(traceback.format_exc())

            return variant_data



import setuptools
import onkopus as op

with open("README.md", "r") as fh:
    long_description = fh.read()
with open("requirements.txt", "r", encoding="utf-8") as fh:
    requirements = fh.read()

version = op.conf_reader.config["DEFAULT"]["VERSION"]

setuptools.setup(
    name="onkopus",
    version=version,
    author="Nadine S. Kurz",
    author_email="nadine.kurz@bioinf.med.uni-goettingen.de",
    description="Variant interpretation framework to analyze and interpret genetic alterations in cancer",
    long_description=long_description,
    long_description_content_type="text/markdown",
    scripts=['./onkopus/onkopus'],
    url="https://gitlab.gwdg.de/MedBioinf/mtb/onkopus/onkopus",
    packages=setuptools.find_packages(),
    install_requires=['requests','numpy','matplotlib','pandas','plotly','scikit-learn','adagenes'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Topic :: Scientific/Engineering :: Medical Science Apps.",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
    python_requires='>=3.9',
    license="GPLv3"
)


import datetime, requests, traceback, copy, io
import adagenes.tools.module_requests as req
from onkopus.conf import read_config as config
import adagenes.tools
from onkopus.onkopus_clients.processing.parse_json import parse_score_results

qid_key = "q_id"
error_logfile=None


class OnkopusServerClient:

    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.info_lines = config.onkopus_server_info_lines
        self.url_pattern_upload = config.onkopus_server_src_upload
        self.srv_prefix = config.onkopus_server_srv_prefix
        self.extract_keys = config.onkopus_server_keys

        self.qid_key = "q_id"
        self.error_logfile = error_logfile

    def get_request(self, request_id):
        """

        :param request_id:
        :return:
        """
        pass

    def interpret_variant_file(self, file_src, genome_version=""):
        """

        :param file_src:
        :return:
        """
        data = {}
        data['data_dir'] = 'loc'

        with open(file_src, 'rb') as file:
            file_content = file.read()
        file.close()
        bytes_io = io.BytesIO(file_content)
        bytes_io.seek(0)
        file = open(file_src,'rb')
        files = { 'file': file }

        #data['file'] = file_content
        data['genome_version'] = genome_version

        print(self.url_pattern_upload)
        qid = requests.post(self.url_pattern_upload, data=data, files=files).text
        file.close()
        print(qid)

        url = config.onkopus_server_src_analyze_id + "?id="+qid
        print(url)
        variant_data = requests.get(url)
        print(variant_data)

    def interpret_variants(self, vcf_lines):
        """

        :param vcf_lines:
        :return:
        """

        qid_list = copy.deepcopy(list(vcf_lines.keys()))
        while True:
            max_length = int(config.config["DEFAULT"]["MODULE_BATCH_SIZE"])
            if max_length > len(qid_list):
                max_length = len(qid_list)
            qids_partial = qid_list[0:max_length]
            variants = ','.join(adagenes.tools.filter_alternate_alleles(qids_partial))

            try:
                json_body = req.get_connection(variants, self.url_pattern, self.genome_version)
                vcf_lines = parse_score_results(vcf_lines, json_body, self.srv_prefix)
            except:
                if error_logfile is not None:
                    cur_dt = datetime.datetime.now()
                    date_time = cur_dt.strftime("%m/%d/%Y, %H:%M:%S")
                    print("error processing request: ", variants, file=error_logfile+str(date_time)+'.log')

            for i in range(0, max_length):
                del qid_list[0]
            if len(qid_list) == 0:
                break

        return vcf_lines



import datetime, traceback, copy
import adagenes.tools.parse_vcf
import adagenes.tools.module_requests as req
from onkopus.conf import read_config as config

qid_key = "q_id"
error_logfile=None

class LoFToolClient:
    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.info_lines = config.loftool_info_lines
        self.url_pattern = config.loftool_src
        self.srv_prefix = config.loftool_srv_prefix
        self.extract_keys = config.loftool_keys

        self.qid_key = "q_id"
        self.error_logfile = None
        #if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
        #    self.qid_key = "q_id_hg19"

    def process_data(self, vcf_lines, input_format='json'):
        if input_format == 'vcf':
            keys = [config.uta_adapter_srv_prefix + config.concat_char + config.uta_genomic_keys[0]]
            annotations = adagenes.tools.parse_vcf.extract_annotations_vcf(vcf_lines, keys)
        else:
            keys = [config.uta_genomic_keys[0]]
            annotations = adagenes.tools.parse_vcf.extract_annotations_json(vcf_lines, config.uta_adapter_srv_prefix, keys)

        #gene_names = annotations[ keys[0] ]
        #genompos_str = ",".join(annotations['q_id'])
        #query = ",".join(gene_names) + '&genompos=' + genompos_str
        #q_ids = annotations['q_id']
        #gene_names = copy.deepcopy(annotations[keys[0]])
        #variant_exchange = copy.deepcopy(annotations[keys[1]])
        #qid_list = copy.deepcopy(annotations['q_id'])
        qid_list = list(vcf_lines.keys())

        while True:
            max_length = int(config.config["DEFAULT"]["MODULE_BATCH_SIZE"])
            if max_length > len(qid_list):
                max_length = len(qid_list)
            qids_partial = qid_list[0:max_length]
            qids_partial = adagenes.tools.filter_alternate_alleles(qids_partial)
            genompos_str = ','.join(qids_partial)
            gene_names_partial = \
                adagenes.tools.parse_vcf.extract_annotations_json_part(vcf_lines, config.uta_adapter_srv_prefix,[config.uta_genomic_keys[0]],
                                                                     qids_partial)[config.uta_genomic_keys[0]]
            gene_names_str = ",".join(gene_names_partial)
            q = 'genesymbol=' + gene_names_str + '&genompos=' + genompos_str

            vcf_lines = adagenes.processing.parse_http_responses.parse_module_response(q, vcf_lines, self.url_pattern,
                                                                                     self.genome_version,
                                                                                     self.srv_prefix)

            for i in range(0,max_length):
                #del gene_names[0] #gene_names.remove(qid)
                #del variant_exchange[0]  #variant_exchange.remove(qid)
                del qid_list[0] # qid_list.remove(qid)
            if len(qid_list) == 0:
                break

        return vcf_lines



import datetime, traceback, copy
import adagenes.tools.module_requests as req
from onkopus.conf import read_config as config
import adagenes.tools


class OncoKBClient:

    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.url_pattern = config.oncokb_src
        self.srv_prefix = config.oncokb_srv_prefix
        self.extract_keys = config.oncokb_keys
        self.info_lines = config.oncokb_info_lines
        self.error_logfile = error_logfile

        self.qid_key = "q_id"
        if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
            self.qid_key = "q_id_hg19"

    def get_headers(self, key):
        """
        Returns the HTTP request header for the OncoKB module

        :return:
        """
        headers = {}
        if key is not None:
            headers["oncokb-key"] = key
            return headers
        else:
            return None

    def process_data(self, vcf_lines, input_format='vcf', key=None):
        """
        Request to OncoKB module

        :param vcf_lines:
        :param input_format:
        :param key
        :return:
        """
        vcf_linesf = adagenes.tools.filter_wildtype_variants(vcf_lines)
        qid_list = copy.deepcopy(list(vcf_linesf.keys()))

        if key is None:
            if "oncokbkey" not in vcf_linesf.keys():
                print("No OncoKB defined")
                return vcf_linesf
            else:
                key = vcf_linesf["oncokbkey"]

        if "oncokbkey" in vcf_linesf.keys():
            vcf_linesf.pop("oncokbkey")
        qid_list.remove("oncokbkey")

        if "oncokbkey" in vcf_linesf.keys():
            key = vcf_linesf["oncokbkey"]
            vcf_linesf.pop("oncokbkey")

        while True:
            max_length = int(config.config["DEFAULT"]["MODULE_BATCH_SIZE"])
            if max_length > len(qid_list):
                max_length = len(qid_list)
            qids_partial = qid_list[0:max_length]

            qids_q = adagenes.tools.filter_alternate_alleles(qids_partial)

            genes = []
            aa_exchanges = []
            for qid in qids_q:
                if qid in vcf_linesf.keys():
                    if "UTA_Adapter" in vcf_linesf[qid]:
                        if "variant_exchange" in vcf_linesf[qid]["UTA_Adapter"]:
                            aa_exchange = vcf_linesf[qid]["UTA_Adapter"]["variant_exchange"]
                            gene = vcf_linesf[qid]["UTA_Adapter"]["gene_name"]
                            genes.append(gene)
                            aa_exchanges.append(aa_exchange)
                    else:
                        genes.append("")
                        aa_exchanges.append("")

            genompos_str = ','.join(qids_q)
            genes_str = ','.join(genes)
            aa_exchange_str = ','.join(aa_exchanges)
            q = ""

            try:
                    q = 'genompos=' + genompos_str + "&genes=" + genes_str + "&aa_exchange=" + aa_exchange_str
                    headers = self.get_headers(key)
                    if headers is None:
                        print("No OncoKB found. Aborting request.")
                        return vcf_lines
                    json_body = req.get_connection(q, self.url_pattern, self.genome_version, headers=headers)

                    for genompos in json_body.keys():
                            json_obj = json_body[genompos]
                            qid = genompos

                            for k in self.extract_keys:
                                if k in json_obj:
                                    pass
                                    #annotations.append('{}_{}={}'.format(self.srv_prefix, k, json_body[i][k]))
                            try:
                                vcf_lines[qid][self.srv_prefix] = json_obj
                            except:
                                if self.error_logfile is not None:
                                    cur_dt = datetime.datetime.now()
                                    date_time = cur_dt.strftime("%m/%d/%Y, %H:%M:%S")
                                    print(cur_dt, ": error processing variant response: ", qid, ';', traceback.format_exc(), file=self.error_logfile+str(date_time)+'.log')
                                else:
                                    print(traceback.format_exc())
            except:
                    if self.error_logfile is not None:
                        print("error processing request: ", vcf_lines, file=self.error_logfile+str(date_time)+'.log')
                    else:
                        print(": error processing variant response: ;", traceback.format_exc())
            for i in range(0, max_length):
                del qid_list[0]
            if len(qid_list) == 0:
                break
        return vcf_lines



import unittest,os
import adagenes
import onkopus


class TreatmentExportTestCase(unittest.TestCase):

    def test_export_treatment_data(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        oncokb_key = os.getenv("ONCOKB_KEY")

        data = {"chr7:140753336A>T":{}, "chr17:7673776G>A":{}}
        genome_version = "hg38"
        bf = adagenes.BiomarkerFrame(genome_version=genome_version,data=data)
        bf = onkopus.annotate(bf, genome_version="hg38",oncokb_key=oncokb_key)

        #print(bf.data["chr7:140753336A>T"]["oncokb"])

        outfile=__location__ + "/../test_files/test_writer.out.csv"
        onkopus.ClinSigWriter().write_evidence_data_to_file(outfile,bf,sep="\t")

        file = open(outfile)
        contents = file.read()
        contents_expected = """biomarker\tcitation_i"""
        self.assertEqual(contents[0:20], contents_expected, "")
        file.close()



    def test_export_treatment_data_filestream(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        oncokb_key = os.getenv("ONCOKB_KEY")

        data = {"chr7:140753336A>T":{}, "chr17:7673776G>A":{}}
        genome_version = "hg38"
        bf = adagenes.BiomarkerFrame(genome_version=genome_version,data=data)
        bf = onkopus.annotate(bf, genome_version="hg38",oncokb_key=oncokb_key)

        #print(bf.data["chr7:140753336A>T"]["oncokb"])

        outfile_str=__location__ + "/../test_files/test_writer.out.stream.csv"
        outfile = open(outfile_str, "w")
        onkopus.ClinSigWriter().write_evidence_data_to_file_all_features(bf,output_file=outfile,sep="\t")
        outfile.close()

        file = open(outfile_str)
        contents = file.read()
        contents_expected = """biomarker\tcitation_i"""
        self.assertEqual(contents[0:20], contents_expected, "")
        file.close()

